/***************************************************************************
 *   Copyright (C) 2005 by Eike Welk   *
 *   eike.welk@post.rwth-aachen.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "siml_globaldef.h"

#include "siml_pygenmain.h"
#include "siml_pyprocessgenerator.h"
#include "siml_code_model.h"

#include <boost/format.hpp>
// #include <boost/tuple/tuple.hpp>
#include "boost/date_time/posix_time/posix_time.hpp"

#include <iostream>
#include <string>
#include <vector>


using std::cout;
using std::string;
using std::vector;

using boost::format;
using namespace boost::posix_time;


siml::PyGenMain::PyGenMain( std::ostream & inPyFile ):
        m_PyFile(inPyFile)
{
}


siml::PyGenMain::~PyGenMain()
{
}


/*!
 Create python objects for all processs in parse_result.
 */
void siml::PyGenMain::generateAll()
{
//     cout << "Start generating Python code --------------------------\n";
    genFileStart();

    //loop over all processs and generate a python object for each.
    //genProcessObject(0); ///@TODO generate all processes
//     cout << "Number of processes: " << repository()->process.size() << "\n";
    CmModelTable::const_iterator it;
    for( it = repository()->process.begin(); it != repository()->process.end(); ++it )
    {
        PyProcessGenerator genProc(m_PyFile);
        genProc.genProcessObject(*it);
        m_PyFile << '\n';
    }
}


/*!
Generate the first few lines of the python file

@todo change from Gnuplot to a more standard plotting library
@todo include name of source file(s)
 */
void siml::PyGenMain::genFileStart()
{
    //get the time
    ptime dateAndTime = second_clock::local_time();

    //output start of file
    m_PyFile <<
            "#!/usr/bin/python\n"
            "\n"
            "#------------------------------------------------------------------------------#\n"
            "#                            Warning: Do not edit!                             #\n"
            "#                                                                              #\n"
            "# This file is generated, it will be overwritten every time the source file(s) #\n"
            "# is/are changed.                                                              #\n"
            "# Write a main routine in an other file. Use import or execfile to load        #\n"
            "# the objects defined in this file into the Python interpreter.                #\n"
            "#------------------------------------------------------------------------------#\n" <<
            format("# Generated by siml version %1% on %2%. %|79t|#\n") % VERSION % dateAndTime;

    //output the list of souce files
    vector< string>::const_iterator itNames = CmCodeRepository::inputFileNames.begin();
    m_PyFile <<     format("# Source file(s): %1%                   %|79t|#\n") % *itNames;
    for( ++itNames; itNames != CmCodeRepository::inputFileNames.end(); ++itNames)
    {
        m_PyFile << format("#                 %1%                   %|79t|#\n") % *itNames;

    }

    //and add some more decorative text
    m_PyFile <<
            "#------------------------------------------------------------------------------#\n"
            "\n"
            "\n"
            ;

    //insert the defion of the base class for all simulators
    m_PyFile <<
    #include "simulatorbase.h"
            ;
}


